#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "stgdk_wc.h"
#include "graphics.h"

float sinMulL[360][64];     // do all muls for rotations before hand
float cosMulL[360][64];

///////////////////////////////////////////////////////////////////////////////
void maketables()
{
  const float PI = 3.141592654;     // value from TI-82
        int   angle,
              count;

  // make trig tables and do muls for faster rotations
  for(angle = 0; angle < 360; angle++)
    for(count = 0; count < 64; count++)
    {
      sinMulL[angle][count] = sin(PI * angle / 180) * (count - 32);
      cosMulL[angle][count] = cos(PI * angle / 180) * (count - 32);
    }
}

///////////////////////////////////////////////////////////////////////////////
void angleput(int putx, int puty, int angle, ptr src, ptr dst)
{
  ptr       pixel;    // pointer to current pixel (incremental)
  byte      color;    // color of current pixel (readability, not needed)
  int        x,  y,   // loop counters
            nx, ny,   // new x and y for current pixel
            xs, ys,   // x and y size of sprite
            cx, cy;   // half of x and y size of sprite
  float     temp;     // rounds off floats from trig tables

  xs = getwid(src);   // get x and y sizes of sprites
  ys = gethgt(src);

  if ((putx + xs >= wminx) && (puty + ys >= wminy) &&
      (putx - xs <= wmaxx) && (puty - ys <= wmaxy))
  {
    cx = xs / 2;        // use x and y sizes with the center
    cy = ys / 2;

    pixel = src + sizeof(src);  // start sprite current pointer at start

    for(y = 0; y < gethgt(src); y++)
    {
      for(x = 0; x < getwid(src); x++)
      {
        color = *pixel;
        pixel++;

        // add the floats from the lookup to the center and round it off
        temp = cosMulL[angle][x - cx + 32] - sinMulL[angle][y - cy + 32];
        if (temp > 0)
          temp += 0.5;
        else
          temp -= 0.5;
        nx = putx + (int)temp;
        temp = cosMulL[angle][y - cy + 32] + sinMulL[angle][x - cx + 32];
        if (temp > 0)
          temp += 0.5;
        else
          temp -= 0.5;
        ny = puty + (int)temp;
      
        // is clipping on and is the point on the screen?
        if ((clipping == FALSE) || 
          ((nx >= wminx) && 
          (nx <= wmaxx) && 
          (ny >= wminy) && 
          (ny <= wmaxy)))
        *(dst + ycoordtable[ny] + nx) = color;
      }
    }
  }
}

///////////////////////////////////////////////////////////////////////////////
void anglexput(int putx, int puty, int angle, ptr src, ptr dst)
{
  ptr       pixel;    // pointer to current pixel (incremental)
  byte      color;    // color of current pixel (readability, not needed)
  int        x,  y,   // loop counters
            nx, ny,   // new x and y for current pixel
            xs, ys,   // x and y size of sprite
            cx, cy;   // half of x and y size of sprite
  float     temp;     // rounds off floats from trig tables

  xs = getwid(src);   // get x and y sizes of sprites
  ys = gethgt(src);
  
  if ((putx + xs >= wminx) && (puty + ys >= wminy) &&
      (putx - xs <= wmaxx) && (puty - ys <= wmaxy))
  {
    cx = xs / 2;        // use x and y sizes with the center
    cy = ys / 2;

    pixel = src + sizeof(src);  // start sprite current pointer at start

    for(y = 0; y < gethgt(src); y++)
    {
      for(x = 0; x < getwid(src); x++)
      {
        color = *pixel;
        pixel++;

        if (color != 0)       // is the pixel transparent?
        {
          // add the floats from the lookup to the center and round it off
          temp = cosMulL[angle][x - cx + 32] - sinMulL[angle][y - cy + 32];
          if (temp > 0)
            temp += 0.5;
          else
            temp -= 0.5;
          nx = putx + (int)temp;
          temp = cosMulL[angle][y - cy + 32] + sinMulL[angle][x - cx + 32];
          if (temp > 0)
            temp += 0.5;
          else
            temp -= 0.5;
          ny = puty + (int)temp;
      
          // is clipping on and is the point on the screen?
          if (((clipping == FALSE) || 
            (nx >= wminx) && 
            (nx <= wmaxx) && 
            (ny >= wminy) && 
            (ny <= wmaxy)))
          *(dst + ycoordtable[ny] + nx) = color;
        }
      }
    }
  }
}

///////////////////////////////////////////////////////////////////////////////
void cline (int x1, int y1, int x2, int y2, byte color)
{
   int x, y;
   float slope = (y1 - y2) / (x1 - x2 + 0.00001);
   
   for (x = x1; x < x1 + abs(x2 - x1); x++) 
   {
      y = slope * (x - x1) + y1;
      if ((x >= wminx) && (x <= wmaxx) && (y >= wminy) && (y <= wmaxy))
        ppix(x, y, color);
   }
}

///////////////////////////////////////////////////////////////////////////////
sprites getspr(char *fname)
{
  // just a little thing so you don't have to declare pal every time
  // FIXME: don't load sprites each time an object is inited
  //        a precache is needed to load all objects once beforehand
  paltype pal[256];

  return loadspr(fname, pal);
}
