#include "stgdk_wc.h"
#include "defs.h"
#include "map.h"

void putclipped(int xloc, int yloc, ptr src, ptr dst);

Map::Map()
{
  int x;
  
  map = NULL;
  spr = NULL;
  for(x = 0; x < MAXYTILES; x++)    // make the y row lookup table
    ylook[x] = x * MAXXTILES;
}

Map::~Map()
{
  if (map != NULL)
    freemap(map);                   // free the map
  if (spr != NULL)
    freespr(spr);                   // free the sprites
}

void Map::Load(char *mapfile, char *sprfile)
{
  paltype pal[256];
  int     temp;

  if (map != NULL)
    freemap(map);                   // free the map
  if (spr != NULL)
    freespr(spr);                   // free the sprites
  
  map = loadmap(mapfile);           // load in the map
  spr = loadspr(sprfile, pal);      // load in the sprites
  
  // scan the map for the edges
  temp = 0;
  while((temp < MAXXTILES) && (map[temp] != NOTILE))
    temp++;  
  maxx = temp;
  temp = 0;
  while((temp < MAXYTILES) && (map[temp * MAXXTILES] != NOTILE))
    temp++;  
  maxy = temp;

  // get the sizes for the sprites
  xsize = getwid(spr[0]);
  ysize = gethgt(spr[0]);

  // precompute how many tiles fit on the screen
  xtiles = (wmaxx / xsize) + 1;
  ytiles = (wmaxy / ysize) + 1;
  
  // fix it to round up, not down, so the screen is always filled
  if (wmaxx % xsize != 0)
    xtiles++;
  if (wmaxy % ysize != 0)
    ytiles++;

  //precompute how many pixels are in the map
  xpix = maxx * xsize;
  ypix = maxy * ysize;
}

void Map::SetLoc(int newx, int newy)
{
  // set the new location for the screen
  scrx = newx;
  scry = newy;
}

void Map::AdjLoc(int xadj, int yadj)
{
  // adjust the location for the screen
  scrx += xadj;
  scry += yadj;
}

void Map::Draw()
{
  // x and y are the tilemap coords
  // tx and ty are the starting tilemap coords
  // ex and ey are the ending tilemap coords
  // sx and sy are the actual screen coords
  // ix and iy are the the starting screen coords
  int x, y,
      tx, ty,
      ex, ey,
      sx, sy,
      ix, iy;

  ix = 0 - (scrx % xsize);
  iy = 0 - (scry % ysize);
  
  tx = (scrx / xsize);
  ty = (scry / ysize);
  
  ex = tx + xtiles;
  ey = ty + ytiles;
  
  sx = ix;
  for(x = tx; x < ex; x++)
  {
    sy = iy;
    for(y = ty; y < ey; y++)
    {
      if ((x >= 0) && (y >= 0) && (x < maxx) && (y < maxy))
        if ((sx < 0) && (sy < 0))
          putclipped(sx, sy, spr[map[x + ylook[y]]], screenaddr);
        else
          solidput(sx, sy, spr[map[x + ylook[y]]], screenaddr);
      sy += ysize;
    }
    sx += xsize;
  }
}

void putclipped(int xloc, int yloc, ptr src, ptr dst)
{
  int x, y;  
  int nx, ny;
  ptr pixel;

  pixel = src + sizeof(src);
  
  for(y = 0; y < gethgt(src); y++)
    for(x = 0; x < getwid(src); x++)
    {
      nx = x + xloc;
      ny = y + yloc;
      if ((nx >= wminx) && 
          (nx <= wmaxx) && 
          (ny >= wminy) && 
          (ny <= wmaxy))
        *(dst + ycoordtable[ny] + nx) = *pixel;
      pixel++;
    }
}

Map world;
