#include "stgdk_wc.h"
#include "defs.h"
#include "graphics.h"
#include "particle.h"
#include "map.h"
#include "subs.h"

///////////////
Particles part;                       // declare particle list for the game
///////////////

Particle::Particle(ParticleData &info)
{
  // initialize particle with proper data
  data = info;
}

int Particle::Update()
{
  // move the particle  
  data.pos.x += xAdj(data.angle, data.speed);
  data.pos.y += yAdj(data.angle, data.speed);

  // update it's color by the decay value
  data.color += data.decay;
  
  // drop it's timer and tell if it's dead
  data.life--;
  if (data.life == 0)
    return FALSE;
  else
    return TRUE;
}

void Particle::Draw()
{
  int x, y;
  
  // compute position on screen relative to world
  x = data.pos.x - world.x();
  y = data.pos.y - world.y();
  
  // if it's visible, draw it on the screen
  if ((x >= wminx) && (x < wmaxx) && (y >= wminy) && (y < wmaxy))
    ppix(x, y, (char)data.color);
}

/////////////////////////////////////////////////////////////////////////////
void Particles::Add(ParticleData &init)
{
  heap[last] = new Particle(init);
  CheckPtr(heap[last]);
  last++;
}

int Particles::Update()
{
  int i;

  for(i = 0; i < last; i++)
    if (heap[i]->Update() == FALSE)
    {
      delete heap[i];
      last--;
      heap[i] = heap[last];
      i--;
    }
  
  if (last == 0)
    return FALSE;
  else
    return TRUE;
}

void Particles::Draw()
{
  int i;

  for(i = 0; i < last; i++)
    heap[i]->Draw();
}

void Particles::Kill()
{
  int i;

  for(i = 0; i < last; i++)
    delete heap[i];
  last = 0; 
}
