#ifndef __STGDK_WC_H
#define __STGDK_WC_H

#ifdef __cplusplus
 extern "C" {
#endif

/***************************************************************************/
/** Redefinitions **********************************************************/
/***************************************************************************/

typedef unsigned char * ptr;
typedef unsigned char byte;
typedef unsigned short word;

#define vm_640x400x256  0x100
#define vm_640x480x256  0x101
#define vm_800x600x256  0x103
#define vm_1024x768x256 0x105

#define TRUE 1
#define FALSE 0

/***************************************************************************/
/** Keyboard Handler Keys **************************************************/
/***************************************************************************/

#define kSPACE        57 // { SPACE BAR }
#define kESC           1 // { ESC }
#define kCTRL         29 // { CONTROL KEY }
#define kALT          56 // { ALTERNATE KEY }
#define kLEFTSHIFT    42 // { LEFT SHIFT }
#define kRIGHTSHIFT   54 // { RIGHT SHIFT }
#define kTILDE        41 // { ~ }
#define kENTER        28 // { ENTER }
#define kCOMMA        51 // { , }
#define kPERIOD       52 // { . }
#define kGRAYPERIOD   83 // { . }
#define kFORWARDSLASH 53 // { / }
#define kUP           72 // { UP }
#define kDOWN         80 // { DOWN }
#define kLEFT         75 // { LEFT }
#define kRIGHT        77 // { RIGHT }
#define kNWDIAG       71 // { KEYPAD 7 }
#define kNEDIAG       73 // { KEYPAD 9 }
#define kSWDIAG       79 // { KEYPAD 1 }
#define kSEDIAG       81 // { KEYPAD 3 }
#define kCENTER       76 // { KEYPAD 5 }
#define kINSERT       82 // { INS }
#define kDELETE       83 // { DEL }
#define kHOME         71 // { HOME }
#define kEND          79 // { END }
#define kPGUP         73 // { Page Up }
#define kPGDN         81 // { Page Down }
#define kMINUS        12 // { - }
#define kGRAYMINUS    74 // { - }
#define kPLUS         13 // { + }
#define kGRAYPLUS     78 // { + }
#define kBACKSLASH    43 // { \ }
#define kBACKSPACE    14 // { BACKSPACE }
#define kTAB          15 // { TAB }
#define kOPENSQUARE   26 // { [ }
#define kCLOSESQUARE  27 // { ] }
#define kSEMICOLON    39 // { ; }
#define kAPOSTROPHE   40 // { ' }
#define kF1           59 // { F1 }
#define kF2           60 // { F2 }
#define kF3           61 // { F3 }
#define kF4           62 // { F4 }
#define kF5           63 // { F5 }
#define kF6           64 // { F6 }
#define kF7           65 // { F7 }
#define kF8           66 // { F8 }
#define kF9           67 // { F9 }
#define kF10          68 // { F10 }
#define kF11          87 // { F11 }
#define kF12          88 // { F12 }
#define k1             2 // { 1 }
#define k2             3 // { 2 }
#define k3             4 // { 3 }
#define k4             5 // { 4 }
#define k5             6 // { 5 }
#define k6             7 // { 6 }
#define k7             8 // { 7 }
#define k8             9 // { 8 }
#define k9            10 // { 9 }
#define k0            11 // { 0 }
#define kA            30 // { A }
#define kB            48 // { B }
#define kC            46 // { C }
#define kD            32 // { D }
#define kE            18 // { E }
#define kF            33 // { F }
#define kG            34 // { G }
#define kH            35 // { H }
#define kI            23 // { I }
#define kJ            36 // { J }
#define kK            37 // { K }
#define kL            38 // { L }
#define kM            50 // { M }
#define kN            49 // { N }
#define kO            24 // { O }
#define kP            25 // { P }
#define kQ            16 // { Q }
#define kR            19 // { R }
#define kS            31 // { S }
#define kT            20 // { T }
#define kU            22 // { U }
#define kV            47 // { V }
#define kW            17 // { W }
#define kX            45 // { X }
#define kY            21 // { Y }
#define kZ            44 // { Z }

/***************************************************************************/
/** File Lib Constants and Variables ***************************************/
/***************************************************************************/

#define bufsize        64000
#define lib_version_major  1
#define lib_version_minor  0
#define NOTINLIB          -1
#define TRUE               1
#define FALSE            0

extern char *libfname;
extern char *libfpass;


#define BASESCREEN (ptr)0xA0000
#define vm_320x200 0x13
#define vm_80x25   0x03

typedef struct
{
	byte R,
			 G,
			 B;
} paltype;


typedef struct
{
	byte manufacturer,
			 version,
			 encode_mode,
			 bits_per_pixel;
	word start_x,
			 start_y,
			 end_x,
			 end_y,
			 horiz_res,
			 vert_res;
	byte palette_rgb[48],
			 RESERVED_BYTE,
			 vmode,
			 planes;
	word bytes_per_line;
	byte RESERVED[59];
} PCXHeader;

typedef struct
{
	word celcode,
			 width,
			 height,
			 hscrpos,
			 vscrpos;
	byte bits,
			 comptype;
	byte reserved[20];
	paltype pal[256];
} CELHeader;


/***************************************************************************/
/** File Formats ***********************************************************/
/***************************************************************************/

struct MNOHeader
{
	word pixel_offset;
	word pixel_repeat;
};


/***************************************************************************/
/** Polygon Specific Variables/Types ***************************************/
/***************************************************************************/

typedef struct
{
	int x,
			y;
} point;

typedef struct
{
	int x,y;
	int sx, sy;   /* Source coordinates from the texture image */
} tpoint;


/***************************************************************************/
/** Lightsourcing Specific Types *******************************************/
/***************************************************************************/

#define MAXSHADES 128L
#define MAXCOLORS 256L
#define SHADESIZE (long)MAXSHADES*(long)MAXCOLORS
#define TABLEHEADER "Light Sourcing Table File\x1A"

typedef byte *shadetbl;


/***************************************************************************/
/** Mouse Related **********************************************************/
/***************************************************************************/

#define MAXBUTTONS 128

#define NOBUTTON 0
#define LEFTBUTTON 1
#define RIGHTBUTTON 2
#define BOTHBUTTONS 3
#define MIDDLEBUTTON 4

typedef struct 
{
	char used;
	int  startx,
			 starty,
			 endx,
			 endy;
} mouse_hotspots;

typedef mouse_hotspots *buthot;

extern int MOUSEBUTTONS;

/***************************************************************************/
/** Variables **************************************************************/
/***************************************************************************/

#define MAXSPRITES 2000

#define MAXXTILES 320
#define MAXYTILES 200
#define MAXTILES  (long)MAXXTILES * (long)MAXYTILES
#define NOTILE    0xFFFF

typedef ptr *sprites;
typedef word *tilemap;

extern byte snowcheck;      /* Snowchecking on or off? (FALSE by default) */
extern byte clipping;       /* Clipping on? */
extern int startx[200];
extern int endx[200];
extern int wminx;           /* min/max values for window set with SETVWIN */
extern int wminy;
extern int wmaxx;
extern int wmaxy;
extern ptr screenaddr;
extern int ycoordtable[200];
extern unsigned char CURRENTCOLOR;

/* vesa stuff */
extern word granmult;
extern long modesize;              // mem req'd for current mode
extern long banksize;
extern long numbanks;
extern byte vesa_currentcolor;
extern char vesa_clipping;

extern int   centerx,
						 centery;
extern float xscale,
						 yscale;


/***************************************************************************/
/** Error Handlers *********************************************************/
/***************************************************************************/

void gfxerror(char *msg);

/***************************************************************************/
/** Some useful CONSOLE IO routines Watcom Lacks ***************************/
/***************************************************************************/

void wc_setactivepage(byte page);
void wc_gotoxy(byte x, byte y, byte page);
byte wc_wherex(byte page);
byte wc_wherey(byte page);
void wc_writech(byte page, word rep, char c, byte attrib);
void wc_clrscr(byte wid, byte hgt); // BE ON PAGE 0 when YOU RUN THIS!

/***************************************************************************/
/** Misc Routines **********************************************************/
/***************************************************************************/

int realtoprot(int seg, int ofs);

/***************************************************************************/
/** File Lib Procedures ****************************************************/
/***************************************************************************/

void openlib(char *libname, char *libpass);
void closelib(void);
int  libopen(void);
void verifypassword(void);
void findfile(char *fname, long *posat, long *size);
void extractfile(char *filetoget);

/***************************************************************************/
/** Mouse Procedures *******************************************************/
/***************************************************************************/

int  msreset(void);
void msgetbutinfo(int button, int *xwhenpressed, int *ywhenpressed,
									int *timespressed);
void msgetpos(word *mx, word *my, word *mb);
void mscurson(void);
void mscursoff(void);
void mssetbounds(word x1, word y1, word x2, word y2);
void mssetpos(word x, word y);
void mssetbmp(ptr bitmap, word hotspotx, word hotspoty);
void mssetmickey(word micksx, word micksy);
byte msbutrel(byte but);
byte msbutprs(byte but);

buthot initbuttons(void);
void createbutton(buthot butinfo, int button,
									word x, word y, word x2, word y2);
void freeallbuttons(buthot butinfo);
int  button_handle(buthot butinfo, word curmx, word curmy);
buthot loadbtn(char *fname);

/***************************************************************************/
/** Keyboard Handler *******************************************************/
/***************************************************************************/

void install_keybhandler(void);
void uninstall_keybhandler(void);
void clearkeys(void);
byte pressed(int key);
byte keybhit(void);

/***************************************************************************/
/** Joystick Procedures ****************************************************/
/***************************************************************************/

void joypoll(int *joyx, int *joyy, int *joyb);
void joycalibrate(void);
byte joypresent(void);

/***************************************************************************/
/** Timer Routines *********************************************************/
/***************************************************************************/

void save_timerint(void);
void restore_timerint(void);
#ifdef __cplusplus
void installtimer(void *isr);         // interrupt removed for use with C++
#else
void installtimer(void interrupt *isr);
#endif
void timer_setpollrate(int xpersec);
float fps(int count, int frames);

/***************************************************************************/
/** Graphics Procedures ****************************************************/
/***************************************************************************/

void setgfxdef(void);
void setvmode(int mode);
byte getvmode(void);
void setvaddr(ptr newaddr);
void genytab(void);
void setvport(int x, int y, int x2, int y2);
void setcolor(byte color);
void ppix(int x, int y, byte color);
byte gpix(int x, int y);
void vclear(void);
void vretrace(void);

/***************************************************************************/
/** Line Drawing, Circles, Rectangles, etc *********************************/
/***************************************************************************/

void line(int startx, int starty, int endx, int endy);
void hline(int startx, int endx, int y);
void vline(int starty, int endy, int x);
void rect(int startx, int starty, int endx, int endy);
void frect(int startx, int starty, int endx, int endy);
void ellipse(int exc, int eyc, int ea, int eb);
void fillellipse(int exc, int eyc, int ea, int eb);
void drawbutton(int x, int y, int x2, int y2,
								int backclr, int highlight, int shadow);


/***************************************************************************/
/** Palette Routines *******************************************************/
/***************************************************************************/

void setpcolor(int color, int r, int g, int b);
void getpal(paltype *p);
void setpal(paltype *p, int stc, int edc);
void blackpal(void);
void fadeout(word dly, int ranges, int rangee);
void fadein(paltype *pal, word dly, int ranges, int rangee);
void cyclepal(paltype *p, int start, int end);

/***************************************************************************/
/** Image Loading Routines *************************************************/
/***************************************************************************/

void loadpal(char *fname, paltype *pal);
void loadcel(char *fname, ptr pl, paltype *p);
void loadpcx(char *fname, ptr pl, paltype *p);
ptr  registermno(char *fname, word allocsize);
void freemno(ptr source);
void showmno(word *source, long picsize, byte drawcolor);
void disppic(ptr image, paltype *pal, int dly, char setthepal);

/***************************************************************************/
/** FLIC Routines **********************************************************/
/***************************************************************************/

void loadfli(char *fname, int speed);
void processnextframe(void);
void showframe(void);
void freefli(void);

/***************************************************************************/
/** Polygons (Tmap, Filled, Outline) ***************************************/
/***************************************************************************/

void polyline(int x1, int y1, int x2, int y2);
void fillpoly(point *vertexlist, int numvertex);
void drawpoly(point *vertexlist, int numvertex);
void tmapline(int x1, int tmapx1, int tmapy1, int x2, int tmapx2,
				int tmapy2, int y, ptr texture);
void tpolyline(int x1, int y1, int tx1, int ty1, int x2, int y2,
				int tx2, int ty2);
void texturedpoly(tpoint *vertexlist, int numvertex, ptr teximg);

/***************************************************************************/
/** Sprite Routines ********************************************************/
/***************************************************************************/

int  getwid(ptr image);
int  gethgt(ptr image);
long imagesize(int x, int y, int x2, int y2);
void getimage(int x, int y, int x2, int y2, ptr src, ptr dst);
void solidput(int putx, int puty, ptr src, ptr dst);
void xrayput(int putx, int puty, ptr src, ptr dst);
void xorput(int putx, int puty, ptr src, ptr dst);
void maskput(int putx, int puty, ptr msk, ptr src, ptr dst);
void maskxrayput(int putx, int puty, ptr msk, ptr src, ptr dst);
void monoput(int putx, int puty, int placecolor, ptr src, ptr dst);
void monoxrayput(int putx, int puty, int placecolor, ptr src, ptr dst);
ptr  scale(ptr source, int neww, int newh);
tilemap loadmap(char *fname);
void    freemap(tilemap tiles);
sprites initspr(void);
sprites loadspr(char *fname, paltype *p);
void    freespr(sprites spr);


/***************************************************************************/
/** Light Sourcing *********************************************************/
/***************************************************************************/

shadetbl lt_loadtbl(char *fname);
void lt_freetbl(shadetbl shadeset);
int  fcc_singlepal(paltype *pal, int color);
int  fcc_doublepal(paltype *pal, paltype *pal2, int color);
void replacecolor(int oldc, int newc);

/***************************************************************************/
/** Text Output ************************************************************/
/***************************************************************************/

void outchar(int xc, int yc, byte asc);
void outtext(int x, int y, char *s);
void outfont(sprites font, int x, int y, char *phrase);
void outfontmono(sprites font, int x, int y, int placecolor, char *phrase);

/***************************************************************************/
/** VESA Routines **********************************************************/
/***************************************************************************/

char vesa_capable(void);
void vesa_switchbank(int bankno);
void vesa_setmode(word mode);
void vesa_setcolor(byte color);
void vesa_setvport(int x, int y, int x2, int y2);
void vesa_clear(void);
void vesa_ppix(long x, long y, byte color);
byte vesa_gpix(long x, long y);
void vesa_line(int startx, int starty, int endx, int endy);
void vesa_getimage(int x, int y, int x2, int y2, ptr dest);
void vesa_solidput(short putx, short puty, ptr source);
void vesa_xrayput(int putx, int puty, ptr source);
void vesa_xorput(short putx, short puty, ptr source);
void vesa_maskput(short putx, short puty, ptr mask, ptr source);
void vesa_maskxrayput(short putx, short puty, ptr mask, ptr source);
void vesa_monoput(short putx, short puty, int placecolor, ptr source);
void vesa_monoxrayput(short putx, short puty, int placecolor, ptr source);
void vesa_loadpcx(char *fname, paltype *p);
void vesa_vtom(ptr source);
void vesa_vclear(ptr virtscr);
void vesa_vloadpcx(char *fname, ptr virtscr, paltype *p);
void vesa_vsolidput(short putx, short puty, ptr source, ptr virtscr);
void vesa_vxrayput(int putx, int puty, ptr source, ptr virtscr);
void vesa_vxorput(short putx, short puty, ptr source, ptr virtscr);
void vesa_vmaskput(short putx, short puty, ptr mask, ptr source, ptr virtscr);
void vesa_vmaskxrayput(short putx, short puty, ptr mask, ptr source, ptr virtscr);
void vesa_vmonoput(short putx, short puty, int placecolor,
									 ptr source, ptr virtscr);
void vesa_vmonoxrayput(short putx, short puty, int placecolor,
									 ptr source, ptr virtscr);
void vesa_outfont(sprites font, int x, int y, char *phrase);
void vesa_outfontmono(sprites font, int x, int y, int placecolor, char *phrase);
void vesa_voutfont(sprites font, ptr dest, int x, int y, char *phrase);
void vesa_voutfontmono(sprites font, ptr dest, int x, int y, int placecolor, char *phrase);

#ifdef __cplusplus
}
#endif

#endif
