#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "stgdk_wc.h"
#include "defs.h"
#include "unitcon.h"
#include "keys.h"
#include "map.h"
#include "particle.h"
#include "weapcon.h"
#include "graphics.h"
#include "subs.h"
#include "text.h"

///////////////
Units units;                        // declare the unit list for the game,
Team  teams[TEAMS];                 // the list of teams,
int   numteams;                     // and how many teams actually exist
///////////////

//*************************************************************************
// quick hack to test for calls to invalid pointers (DOS4GW should error?)
// FIXME:  remove...
class Junk : public Unit
{
  public:
    Junk(UnitData &info);
    void PreThink() { gfxerror("Junk:  PreThink was called!"); }
    void PostThink() { gfxerror("Junk:  PostThink was called!"); }
    void TakeDamage() { gfxerror("Junk:  TakeDamage was called!"); }
};

Junk::Junk(UnitData &info):
Unit(info, "borg.spr") 
{ 
  data.shields = 200;
  data.hull = 50;
  data.collision = 50;
  data.classname = C_BORG;
  data.pos.x = 5120;
  data.pos.y = 3200;
  data.angle = 0;
  data.speed = 0;
  data.team  = T_PLAYER;
}

UnitData junkdata;
Unit *junk = new Junk(junkdata);
//*************************************************************************

/////////////////////////////////////////////////////////////////////////////
Unit::Unit(UnitData &info, char *sprfile)
{
  data = info;
  
  data.spr = getspr(sprfile);
  data.xsize = getwid(data.spr[0]);
  data.ysize = gethgt(data.spr[0]);
  data.bxsize = data.xsize * 0.50;     // make box bigger than sprite so when
  data.bysize = data.ysize * 0.50;     // it's angled collisions are better
  data.current = 0;
  data.drawangle = data.angle;
  
  data.nextfire = 0;

  data.dead = FALSE;
}

///////////////
virtual Unit::~Unit()
{
  Explode();  

  freespr(data.spr);
}

///////////////
void Unit::Explode()
{
  // this takes the pixels of the unit and turnes them into
  // particles moving in random directions/speed 
  // it basically a hacked angleput()
  // instead of drawing it adds them to the particle list
  // it should send them out in their relative angle to their
  // position in the bitmap (like scaling the pixels up)
  // FIXME:  move out of class
  // FIXME:  this code is slow...4 divides PER PIXEL?!
  ptr       pixel;    // pointer to current pixel (incremental)
  int        x,  y,   // loop counters
            cx, cy;   // half of x and y size of sprite
  float     temp;     // rounds off floats from trig tables
  ParticleData init;  // starting data for particle of the sprite

  cx = data.xsize / 2; // use x and y sizes with the center
  cy = data.ysize / 2;

  pixel = data.spr[data.current] + sizeof(sprites);  // start sprite current pointer at start

  for(y = 0; y < data.ysize; y++)
  {
    for(x = 0; x < data.xsize; x++)
    {
      if (*pixel != 0)
      {
        // add the floats from the lookup to the center and round it off
        temp = cosMulL[data.drawangle][x - cx + 32] - 
                sinMulL[data.drawangle][y - cy + 32];
        if (temp > 0)
          temp += 0.5;
        else
            temp -= 0.5;
        init.pos.x = data.pos.x + (int)temp;
        temp = cosMulL[data.angle][y - cy + 32] + 
                sinMulL[data.angle][x - cx + 32];
        if (temp > 0)
          temp += 0.5;
        else
            temp -= 0.5;
        init.pos.y = data.pos.y + (int)temp;
        init.angle = rand() % 360;
        init.color = *pixel;
        init.speed = (rand() % 6) + 2;
        init.life = ((rand() % 35) + 25) / init.speed;
        init.decay = 0;
        part.Add(init);
      }
      pixel++;
    }
  }
}

///////////////
void Unit::Update()
{
  data.pos.x += xAdj(data.angle, data.speed);
  data.pos.y += yAdj(data.angle, data.speed);

  if ((data.pos.x < 0) || (data.pos.x >= world.xmax())) 
  {
    data.angle = 360 - data.angle;
    FixAngle(data.angle);
    data.pos.x += xAdj(data.angle, data.speed);
    data.pos.y += yAdj(data.angle, data.speed);
  }
  if ((data.pos.y < 0) || (data.pos.y >= world.ymax())) 
  {
    data.angle = 180 - data.angle;
    FixAngle(data.angle);
    data.pos.x += xAdj(data.angle, data.speed);
    data.pos.y += yAdj(data.angle, data.speed);
  }
  
  data.boxmin.x = data.pos.x - data.bxsize;
  data.boxmin.y = data.pos.y - data.bysize;
  data.boxmax.x = data.pos.x + data.bxsize;
  data.boxmax.y = data.pos.y + data.bysize;

  data.nextfire--;
  if (data.nextfire < 0)
    data.nextfire = 0;
}

///////////////
void Unit::Draw()
{
    anglexput(data.pos.x - world.x(), data.pos.y - world.y(), 
              data.drawangle, data.spr[data.current], screenaddr);
}

///////////////
void Unit::Damage(int damage, Unit *attacker, int team)
{
  ParticleData init;
           int angle;

  if (data.shields > 0)           // if shields exist, draw shield bubble
  {
    for(angle = 0; angle < 360; angle += 3)
    {
      init.pos.x = data.pos.x + xAdj(angle, data.bxsize * 1.5);
      init.pos.y = data.pos.y + yAdj(angle, data.bysize * 1.5);
      init.color = 60 + (rand() % 10);
      init.angle = 0;
      init.speed = 0;
      init.life = 1;
      init.decay = 0;
      part.Add(init);
    }
  }
  data.shields -= damage;         // put all damage through shields
  if (data.shields < 0)           // are the shields are overkilled?
  {
    data.hull += data.shields;    // use the excessive damage on the hull
    data.shields = 0;             // reset shields to 0
    if (data.hull > 0)            // anything left?  if not, shoot sparks
    {
      for(angle = 0; angle < 10; angle++)
      {
        init.pos.x = data.pos.x;
        init.pos.y = data.pos.y;
        init.color = 63 + (rand() % 7);
        init.angle = rand() % 360;
        init.speed = (rand() % 4) + 1;
        init.life = (rand() % 6) + 2;
        init.decay = 0;
        part.Add(init);
      }
    }
  }

  data.attacker = attacker;       // set last attacker to current one
  
  TakeDamage();                   // let the individual unit know about it
  
  if ((data.hull <= 0) && (data.dead == FALSE))   // are we dead?
    Killed(team);                                 // better clean up then
}

///////////////
void Unit::Killed(int team)
{
  char message[255];

  data.dead = TRUE;                               // mark for deletion

  if (team != data.team)                          // update kills
    teams[team].kills++;

  strcpy(message, teams[data.team].name);         // print obituary   
  switch (rand() % 5)
  {
    case 0: 
      strcat(message, " was killed by ");
      break;
    case 1: 
      strcat(message, " was toasted by ");
      break;
    case 2:
      strcat(message, " lost to ");
      break;
    case 3:
      strcat(message, " got caught by ");
      break;
    case 4:
      strcat(message, " was just zapped by ");
      break;
  }
  strcat(message, teams[team].name);
  text.Print(message);
}

///////////////
void Unit::NextFire(int next)
{
  data.nextfire += next;
}

///////////////
int Unit::Fire()
{
  if (data.nextfire <= 0)
    return TRUE;
  else
    return data.nextfire;
}

///////////////
void Unit::Check()
{
  Unit *list;
  point bmin, bmax;

  list = units.ChainAll();
  while (list != NULL)
  {
    bmin = list->boxmin();
    bmax = list->boxmax();
    if ((list != this) &&
        (bmin.x <  data.boxmax.x) &&
        (bmax.x >= data.boxmin.x) &&
        (bmin.y <  data.boxmax.y) &&
        (bmax.y >= data.boxmin.y))
    {
      if ((abs(FindAngle(data.pos, list->pos()) - data.angle) <= 90) &&
          (data.speed > 0))
      {
        data.angle = FindAngle(data.pos, list->pos()) - 180;
        FixAngle(data.angle);
      }
      data.pos.x += xAdj(data.angle, data.speed + 5);
      data.pos.y += yAdj(data.angle, data.speed + 5);
      Damage(list->collision(), list, list->team());
    }
    list = units.ChainNext();
  }
}


///////////////
Unit *Unit::Closest()
{
  // FIXME:  should be better optimized...only comp dist once per time
  int  dist;
  Unit *temp,
       *closest;

  dist = 0xFFFF;
  closest = NULL;
  
  temp = units.ChainAll();
  while (temp != NULL)
  {
    if ((temp != this) && (Distance(data.pos, temp->pos()) <= dist))
    {
      dist = Distance(data.pos, temp->pos());
      closest = temp;
    }
    temp = units.ChainNext();
  }

  return closest;
}

///////////////
Unit *Unit::ClosestNTeam()
{
  // FIXME:  should be better optimized...only comp dist once per time
  int  dist;
  Unit *temp,
       *closest;

  dist = 0xFFFF;
  closest = NULL;
  
  temp = units.ChainNTeam(data.team);
  while (temp != NULL)
  {
    if ((temp != this) && (Distance(data.pos, temp->pos()) <= dist))
    {
      dist = Distance(data.pos, temp->pos());
      closest = temp;
    }
    temp = units.ChainNext();
  }

  return closest;
}

///////////////
void Unit::SetTeam(int teamnum)
{
  data.team = teamnum;
}

/////////////////////////////////////////////////////////////////////////////
// Note:  all this used to use a linked list, now uses an array
/*
=================
List and Unit Management:
=================
*/
void Units::Add(Unit *unit)
{
  // NOTE:  this assumes that the pointer passed is valid!
  heap[last] = unit;
  last++;
}

///////////////
int Units::Update()
{
  int  i;  

  for(i = 0; i < last; i++)
    heap[i]->PreThink();
  
  for(i = 0; i < last; i++)
    heap[i]->Update();

  for(i = 0; i < last; i++)
    heap[i]->Check();
  
  weapons.Update();
  
  for(i = 0; i < last; i++)
    if (heap[i]->dead() == TRUE)
    {
      delete heap[i];    
      last--;
      heap[i] = heap[last];
      i--;
    }
    else
      if (heap[i]->Fire() == TRUE)
        heap[i]->PostThink();
  
  if (last == 0)
    return FALSE;
  else
    return TRUE;
}

///////////////
void Units::Draw()
{
  int i;  

  for(i = 0; i < last; i++)
    heap[i]->Draw();
}

/*
=================
Chain Functions:
=================
*/

///////////////
Unit *Units::ChainNext()
{
  curr++;

  if (curr == maxchain)
    return NULL;
  else
    return chain[curr];
}

///////////////
Unit *Units::ChainAll()
{
  for(maxchain = 0; maxchain < last; maxchain++)
    chain[maxchain] = heap[maxchain];
  
  curr = 0;
  if (maxchain == 0)
    return NULL;
  else
    return chain[0];
}

///////////////
Unit *Units::ChainRadius(point origin, int radius)
{
  for(maxchain = 0; maxchain < last; maxchain++)
    if (Distance(origin, heap[maxchain]->pos()) <= radius)
      chain[maxchain] = heap[maxchain];

  curr = 0;
  if (maxchain == 0)
    return NULL;
  else
    return chain[0];
}

///////////////
Unit *Units::ChainTeam(int team)
{
  for(maxchain = 0; maxchain < last; maxchain++)
    if (team == heap[maxchain]->team())
      chain[maxchain] = heap[maxchain];

  curr = 0;
  if (maxchain == 0)
    return NULL;
  else
    return chain[0];
}

///////////////
Unit *Units::ChainNTeam(int team)
{
  for(maxchain = 0; maxchain < last; maxchain++)
    if (team != heap[maxchain]->team())
      chain[maxchain] = heap[maxchain];

  curr = 0;
  if (maxchain == 0)
    return NULL;
  else
    return chain[0];
}

///////////////
Unit *Units::ChainClassname(int name)
{
  for(maxchain = 0; maxchain < last; maxchain++)
    if (name == heap[maxchain]->classname())
      chain[maxchain] = heap[maxchain];

  curr = 0;
  if (maxchain == 0)
    return NULL;
  else
    return chain[0];
}

///////////////
int Units::Exist(Unit *unit)
{
  int i;  

  for(i = 0; i < last; i++)
    if (heap[i] == unit)
      return TRUE;
  return FALSE;
}

///////////////
void Units::Kill()
{
  int i;  

  for(i = 0; i < last; i++)
    delete heap[i];
  last = 0;
}
