#include <stdlib.h>
#include "stgdk_wc.h"
#include "defs.h"
#include "unitcon.h"
#include "map.h"
#include "weapcon.h"
#include "subs.h"
#include "graphics.h"

////////////////
Weapons weapons;                  // declare the weapons list for the game
////////////////

/////////////////////////////////////////////////////////////////////////////
int Weapon::Update()
{
  data.pos.x += xAdj(data.angle, data.speed);
  data.pos.y += yAdj(data.angle, data.speed);

  if ((data.pos.x < 0) || (data.pos.y < 0) ||
      (data.pos.x >= world.xmax()) || (data.pos.y >= world.ymax()))
    data.time = 0;
  
  data.time--;                      // drop it's timer
  if (data.time <= 0)               // if it's dead, remove it
    return FALSE;
  else
    return TRUE;
}

///////////////
void Weapon::Check()
{
  point boxmin,
        boxmax;
  Unit  *temp;

  temp = units.ChainAll();
  while (temp != NULL)
  {
    boxmin = temp->boxmin();
    boxmax = temp->boxmax();
    if ((data.pos.x >= boxmin.x) && (data.pos.y >= boxmin.y) &&
        (data.pos.x <= boxmax.x) && (data.pos.y <= boxmax.y) &&
        (temp != data.owner))
    {
      data.time = 0;
      Impact(temp);
      return;
    }
    temp = units.ChainNext();
  }
}

///////////////
void Weapon::Draw()
{
  int x, y;
  
  x = data.pos.x - world.x();   // compute position on screen
  y = data.pos.y - world.y();   // relative to world
  
  anglexput(x, y, data.angle, data.spr[data.curr], screenaddr);
}

///////////////
Unit *Weapon::Closest()
{
  // FIXME:  should be better optimized...only comp dist once per time
  int  dist;
  Unit *temp,
       *closest;

  dist = 0xFFFF;
  closest = NULL;
  
  temp = units.ChainAll();
  while (temp != NULL)
  {
    if ((temp != data.owner) && (Distance(data.pos, temp->pos()) <= dist))
    {
      dist = Distance(data.pos, temp->pos());
      closest = temp;
    }
    temp = units.ChainNext();
  }

  return closest;
}

/////////////////////////////////////////////////////////////////////////////
void Weapons::Add(Weapon *init)
{
  heap[last] = init;
  heap[last]->Update();
  last++;
}

///////////////
int Weapons::Update()
{
  int i;

  for(i = 0; i < last; i++)
    heap[i]->PreThink();

  for(i = 0; i < last; i++)
    if (heap[i]->Update() == FALSE)
    {
      delete heap[i];
      last--;
      heap[i] = heap[last];
      i--;
    }
  
  for(i = 0; i < last; i++)
  {
    heap[i]->Check();
    heap[i]->PostThink();
  }

  if (last == 0)
    return FALSE;
  else
    return TRUE;
}

///////////////
void Weapons::Draw()
{
  int i;

  for(i = 0; i < last; i++)
    heap[i]->Draw();
}

///////////////
void Weapons::Kill()
{
  int i;

  for(i = 0; i < last; i++)
    delete heap[i];
  last = 0;
}
