#ifndef __weapcon_h
#define __weapcon_h

typedef struct
{
  point   pos;      // location in world
  int     angle,    // angle in degrees (only used with speed)
          speed,    // speed in pixels/frame
          curr;     // sprite to use for drawing
  int     time;     // time until it expires (0 == expired)
  sprites spr;      // pointer to weapon's sprite list
  Unit    *owner;   // unit that fired it (won't collide with owner)
  int     team;     // team that it belongs to
} WeaponData;

class Weapon
{
  public:
    Weapon() {};
    virtual ~Weapon() {};
    
    void virtual PreThink() = 0;     // setup for movement
    int  Update();                   // move object and do all checks
    void Check();                    // check all units for collision
    void virtual PostThink() = 0;    // spawn other weapons/units

    void Draw();                     // draw the weapon on screen

    void virtual Impact(Unit *hit) = 0;  // called when collided
  
    Unit *Closest();                 // closest unit to itself

  protected:
    WeaponData data;
};

class Weapons
{
  public:
    Weapons() { last = 0; }
    ~Weapons() {};
    
    void Add(Weapon *init);
    int  Update();
    void Draw();
    
    void Kill();
    void Kill(Weapon *temp);

  protected:
    Weapon *heap[8192];
    int    last;
};


extern Weapons weapons;

#endif
