#include <stdlib.h>
#include "stgdk_wc.h"
#include "defs.h"
#include "subs.h"
#include "graphics.h"
#include "unitcon.h"
#include "weapcon.h"
#include "weapons.h"
#include "particle.h"

/////////////////////////////////////////////////////////////////////////////
LaserCannon::LaserCannon(int x, int y, int angle, Unit *owner, int team):Weapon()
{
  data.pos.x = x;
  data.pos.y = y;
  data.angle = angle;
  data.owner = owner;
  if (units.Exist(data.owner))
    data.speed = owner->speed() + 22;
  else
    data.speed = 22;
  data.time = 10;
  data.curr = 0;
  data.team = team;
  data.spr = getspr("laser.spr");
  if (units.Exist(data.owner))
    owner->NextFire(2);
  weapons.Add(this);
}

///////////////
void LaserCannon::PreThink() {}

///////////////
void LaserCannon::PostThink() {}

///////////////
void LaserCannon::Impact(Unit *hit)
{
  data.time = 0;
  hit->Damage(5, data.owner, data.team);
}

/////////////////////////////////////////////////////////////////////////////
PlasmaCannon::PlasmaCannon(int x, int y, int angle, Unit *owner, int team):Weapon()
{
  data.pos.x = x;
  data.pos.y = y;
  data.angle = angle;
  data.owner = owner;
  if (units.Exist(data.owner))
    data.speed = owner->speed() + 20;
  else
    data.speed = 20;
  data.time = 15;
  data.curr = 0;
  data.team = team;
  data.spr = getspr("plasma.spr");
  if (units.Exist(data.owner))
    owner->NextFire(2);
  weapons.Add(this);
}

///////////////
void PlasmaCannon::PreThink() {}

///////////////
void PlasmaCannon::PostThink() {}

///////////////
void PlasmaCannon::Impact(Unit *hit)
{
  data.time = 0;
  hit->Damage(8, data.owner, data.team);
}

/////////////////////////////////////////////////////////////////////////////
MiniLaser::MiniLaser(int x, int y, int angle, Unit *owner, int team):Weapon()
{
  data.pos.x = x;
  data.pos.y = y;
  data.angle = angle;
  data.owner = owner;
  if (units.Exist(data.owner))
    data.speed = owner->speed() + 18;
  else
    data.speed = 18;
  data.time = 15;
  data.curr = 0;
  data.team = team;
  data.spr = getspr("minilasr.spr");
  if (units.Exist(data.owner))
    owner->NextFire(2);
  weapons.Add(this);
}

///////////////
void MiniLaser::PreThink() {}

///////////////
void MiniLaser::PostThink() {}

///////////////
void MiniLaser::Impact(Unit *hit)
{
  data.time = 0;
  hit->Damage(3, data.owner, data.team);
}

/////////////////////////////////////////////////////////////////////////////
DisableCannon::DisableCannon(int x, int y, int angle, Unit *owner, int team):Weapon()
{
  data.pos.x = x;
  data.pos.y = y;
  data.angle = angle;
  data.owner = owner;
  if (units.Exist(data.owner))
    data.speed = owner->speed() + 18;
  else
    data.speed = 18;
  data.time = 15;
  data.curr = 0;
  data.team = team;
  data.spr = getspr("disable.spr");
  if (units.Exist(data.owner))
    owner->NextFire(2);
  weapons.Add(this);
}

///////////////
void DisableCannon::PreThink() {}

///////////////
void DisableCannon::PostThink() {}

///////////////
void DisableCannon::Impact(Unit *hit)
{
  data.time = 0;
  hit->NextFire(20);
}

/////////////////////////////////////////////////////////////////////////////
MiniMissile::MiniMissile(int x, int y, int angle, Unit *owner, int team):Weapon()
{
  data.pos.x = x;
  data.pos.y = y;
  data.angle = angle;
  data.owner = owner;
  if (units.Exist(data.owner))
    data.speed = owner->speed() + 15;
  else
    data.speed = 15;
  data.time = 30;
  data.curr = 0;
  data.team = team;
  data.spr = getspr("minimisl.spr");
  if (units.Exist(data.owner))
    owner->NextFire(7);
  weapons.Add(this);
}

///////////////
void MiniMissile::PreThink() {}

///////////////
void MiniMissile::PostThink() 
{
  int angle;
  ParticleData init;

  for(angle = 120; angle < 240; angle += 30)
  {
    init.pos.x = data.pos.x + xAdj(data.angle + angle, (rand() % 7));
    init.pos.y = data.pos.y + yAdj(data.angle + angle, (rand() % 7));
    init.pos.x += xAdj(data.angle + 180, 6 + (rand() % 12));
    init.pos.y += yAdj(data.angle + 180, 6 + (rand() % 12));
    init.color = 120 + (rand() % 7);
    init.angle = 0;
    init.speed = 0;
    init.life = 1;
    init.decay = 0;
    part.Add(init);
  }
}

///////////////
void MiniMissile::Impact(Unit *hit)
{
  data.time = 0;
  hit->Damage(10, data.owner, data.team);
}

/////////////////////////////////////////////////////////////////////////////
DumbFireMissile::DumbFireMissile(int x, int y, int angle, Unit *owner, int team):Weapon()
{
  data.pos.x = x;
  data.pos.y = y;
  data.angle = angle;
  data.owner = owner;
  if (units.Exist(data.owner))
    data.speed = owner->speed() + 12;
  else
    data.speed = 12;
  data.time = 35;
  data.curr = 0;
  data.team = team;
  data.spr = getspr("dumbfire.spr");
  if (units.Exist(data.owner))
    owner->NextFire(15);
  weapons.Add(this);
}

///////////////
void DumbFireMissile::PreThink() {}

///////////////
void DumbFireMissile::PostThink() 
{
  int angle;
  ParticleData init;

  for(angle = 120; angle < 240; angle += 15)
  {
    init.pos.x = data.pos.x + xAdj(data.angle + angle, (rand() % 7));
    init.pos.y = data.pos.y + yAdj(data.angle + angle, (rand() % 7));
    init.pos.x += xAdj(data.angle + 180, 6 + (rand() % 12));
    init.pos.y += yAdj(data.angle + 180, 6 + (rand() % 12));
    init.color = 56 + (rand() % 8);
    init.angle = 0;
    init.speed = 0;
    init.life = 1 + (rand() % 2);
    init.decay = 0;
    part.Add(init);
  }
}

///////////////
void DumbFireMissile::Impact(Unit *hit)
{
  ParticleData init;
  Unit         *temp;
  int          x,
               damage;
  
  data.time = 0;

  hit->Damage(75, data.owner, data.team);

  temp = units.ChainRadius(data.pos, 70);
  while (temp != NULL)
  {
    if (temp != hit)        // already took damage from impact
    {
      damage = (70 - Distance(temp->pos(), data.pos)) / 2;
      temp->Damage(damage, data.owner, data.team);
    }
    temp = units.ChainNext();
  }
 
 for(x = 0; x < 150; x++)
  {
    init.pos.x = data.pos.x;
    init.pos.y = data.pos.y;
    init.color = 16 + (rand() % 12);
    init.angle = (rand() % 360);
    init.speed = (rand() % 10) + 5;
    init.life = (70 / init.speed) + 1;
    init.decay = 0;
    part.Add(init);
  }
}

/////////////////////////////////////////////////////////////////////////////
Mirv::Mirv(int x, int y, int angle, Unit *owner, int team):Weapon()
{
  data.pos.x = x;
  data.pos.y = y;
  data.angle = angle;
  data.owner = owner;
  if (units.Exist(data.owner))
    data.speed = owner->speed() + 5;
  else
    data.speed = 5;
  data.time = 15;
  data.curr = 0;
  data.team = team;
  data.spr = getspr("mirv.spr");
  if (units.Exist(data.owner))
    owner->NextFire(15);
  weapons.Add(this);
}

///////////////
void Mirv::PreThink() {}

///////////////
void Mirv::PostThink() 
{
  if (data.time == 1)
  {
    new MiniMissile(data.pos.x, data.pos.y, data.angle - 10, data.owner, data.team);
    new MiniMissile(data.pos.x, data.pos.y, data.angle, data.owner, data.team);
    new MiniMissile(data.pos.x, data.pos.y, data.angle + 10, data.owner, data.team);
    if (units.Exist(data.owner))
      data.owner->NextFire(-21);
    data.time = 0;
  }
}

///////////////
void Mirv::Impact(Unit *hit)
{
  data.time = 0;
  hit->Damage(10, data.owner, data.team);
}

/////////////////////////////////////////////////////////////////////////////
HomingMissile::HomingMissile(int x, int y, int angle, Unit *owner, int team):Weapon()
{
  data.pos.x = x;
  data.pos.y = y;
  data.angle = angle;
  data.owner = owner;
  if (units.Exist(data.owner))
    data.speed = owner->speed() + 10;
  else
    data.speed = 10;
  data.time = 100;
  data.curr = 0;
  data.team = team;
  data.spr = getspr("homing.spr");
  if (units.Exist(data.owner))
    owner->NextFire(15);
  weapons.Add(this);
}

///////////////
void HomingMissile::PreThink() 
{
  Unit *unit;

  unit = Closest();
  if (unit != NULL)
    MoveAngle(data.angle, FindAngle(data.pos, unit->pos()), 8);
}

///////////////
void HomingMissile::PostThink() 
{
  int angle;
  ParticleData init;

  for(angle = 120; angle < 240; angle += 30)
  {
    init.pos.x = data.pos.x + xAdj(data.angle + angle, (rand() % 7));
    init.pos.y = data.pos.y + yAdj(data.angle + angle, (rand() % 7));
    init.pos.x += xAdj(data.angle + 180, 6 + (rand() % 12));
    init.pos.y += yAdj(data.angle + 180, 6 + (rand() % 12));
    init.color = 56 + (rand() % 8);
    init.angle = 0;
    init.speed = 0;
    init.life = 1 + (rand() % 2);
    init.decay = 0;
    part.Add(init);
  }
}

///////////////
void HomingMissile::Impact(Unit *hit)
{
  data.time = 0;
  hit->Damage(15, data.owner, data.team);
}

/////////////////////////////////////////////////////////////////////////////
SmartMissile::SmartMissile(int x, int y, int angle, Unit *owner, int team):Weapon()
{
  data.pos.x = x;
  data.pos.y = y;
  data.angle = angle;
  data.owner = owner;
  if (units.Exist(data.owner))
    data.speed = owner->speed() + 15;
  else
    data.speed = 15;
  data.time = 100;
  data.curr = 0;
  data.team = team;
  data.spr = getspr("smartmsl.spr");
  if (units.Exist(data.owner))
    owner->NextFire(20);
  weapons.Add(this);
}

///////////////
void SmartMissile::PreThink()
{
  Unit *unit;

  unit = Closest();
  if (unit != NULL)
    MoveAngle(data.angle, FindAngle(data.pos, unit->pos()), 20);
}

///////////////
void SmartMissile::PostThink() 
{
  int angle;
  ParticleData init;

  for(angle = 120; angle < 240; angle += 30)
  {
    init.pos.x = data.pos.x + xAdj(data.angle + angle, (rand() % 7));
    init.pos.y = data.pos.y + yAdj(data.angle + angle, (rand() % 7));
    init.pos.x += xAdj(data.angle + 180, 6 + (rand() % 12));
    init.pos.y += yAdj(data.angle + 180, 6 + (rand() % 12));
    init.color = 56 + (rand() % 8);
    init.angle = 0;
    init.speed = 0;
    init.life = 1 + (rand() % 2);
    init.decay = 0;
    part.Add(init);
  }
}

///////////////
void SmartMissile::Impact(Unit *hit)
{
  data.time = 0;
  hit->Damage(25, data.owner, data.team);
}

/////////////////////////////////////////////////////////////////////////////
BlastBeam::BlastBeam(int x, int y, int angle, Unit *owner, int team):Weapon()
{
  data.pos.x = x;
  data.pos.y = y;
  data.angle = angle;
  data.owner = owner;
  if (units.Exist(data.owner))
    data.speed = owner->speed() + 8;
  else
    data.speed = 8;
  data.time = 50;
  data.curr = 0;
  data.team = team;
  data.spr = getspr("suprbeam.spr");
  if (units.Exist(data.owner))
    owner->NextFire(50);
  weapons.Add(this);
}

///////////////
void BlastBeam::PreThink() {}

///////////////
void BlastBeam::PostThink() 
{
  int angle;
  ParticleData init;

  for(angle = 135; angle < 225; angle += 15)
  {
    init.pos.x = data.pos.x + xAdj(data.angle + angle, (rand() % 7));
    init.pos.y = data.pos.y + yAdj(data.angle + angle, (rand() % 7));
    init.pos.x += xAdj(data.angle + 180, 2 + (rand() % 14));
    init.pos.y += yAdj(data.angle + 180, 2 + (rand() % 14));
    init.color = 224 + (rand() % 15);
    init.angle = 0;
    init.speed = 0;
    init.life = 1 + (rand() % 5);
    init.decay = 0;
    part.Add(init);
  }
}

///////////////
void BlastBeam::Impact(Unit *hit)
{    
  int  dist;
  Unit *temp,
       *closest,
       *last;
  
  data.time = 0;
  hit->Damage(10000, data.owner, data.team);

  // fire at the closest if not the owner
  dist = 0xFFFF;
  closest = NULL;
  temp = units.ChainNTeam(data.team);
  while (temp != NULL)
  {
    if ((temp != hit) && (temp != data.owner) &&
        (Distance(data.pos, temp->pos()) <= dist))
    {
      dist = Distance(data.pos, temp->pos());
      closest = temp;
    }
    temp = units.ChainNext();
  }
  if (closest != NULL)
  {
    new BlastBeam(data.pos.x, data.pos.y, FindAngle(data.pos, 
                  closest->pos()), data.owner, data.team);
    if (units.Exist(data.owner))
      data.owner->NextFire(-25);      // 50% penalty for spawns
  }
  
  // fire at the next closest if not the owner or closest
  dist = 0xFFFF;
  last = closest;
  closest = NULL;
  temp = units.ChainNTeam(data.team);
  while (temp != NULL)
  {
    if ((temp != hit) && (temp != data.owner) && (temp != last) &&
        (Distance(data.pos, temp->pos()) <= dist))
    {
      dist = Distance(data.pos, temp->pos());
      closest = temp;
    }
    temp = units.ChainNext();
  }
  if (closest != NULL)
  {
    new BlastBeam(data.pos.x, data.pos.y, FindAngle(data.pos, 
                  closest->pos()), data.owner, data.team);
    if (units.Exist(data.owner))
      data.owner->NextFire(-40);      // 50% penalty for spawns
  }
}

/////////////////////////////////////////////////////////////////////////////
